@@ -25,6 +25,8 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include "ignition/common/Util.hh"
+
 // The symlink tests should always work on UNIX systems
 #define BUILD_SYMLINK_TESTS
 
@@ -32,12 +34,8 @@
 bool create_and_switch_to_temp_dir(std::string &_new_temp_path)
 {
   std::string tmppath;
-  const char *tmp = getenv("TMPDIR");
-  if (tmp)
-  {
-    tmppath = std::string(tmp);
-  }
-  else
+
+  if (!ignition::common::env("TMPDIR", tmppath))
   {
     tmppath = std::string("/tmp");
   }
@@ -109,7 +107,7 @@ bool create_new_file_hardlink(const std::string &_hardlink,
 #ifdef IGN_BUILD_SYMLINK_TESTS_ON_WINDOWS
 // The symlink tests require special permissions to work on Windows,
 // so they will be disabled by default. For more information, see:
-// https://bitbucket.org/ignitionrobotics/ign-common/issues/21
+// https://github.com/ignitionrobotics/ign-common/issues/21
 #define BUILD_SYMLINK_TESTS
 #endif
 
@@ -154,13 +152,19 @@ bool create_and_switch_to_temp_dir(std::string &_new_temp_path)
 /////////////////////////////////////////////////
 bool create_new_empty_file(const std::string &_filename)
 {
-  return ::CreateFileA(_filename.c_str(),
-                       FILE_READ_DATA,
-                       FILE_SHARE_READ,
-                       nullptr,
-                       OPEN_ALWAYS,
-                       0,
-                       nullptr) != INVALID_HANDLE_VALUE;
+  const auto handle = ::CreateFileA(_filename.c_str(),
+                                    FILE_READ_DATA,
+                                    FILE_SHARE_READ,
+                                    nullptr,
+                                    OPEN_ALWAYS,
+                                    0,
+                                    nullptr);
+  if (handle != INVALID_HANDLE_VALUE)
+  {
+    ::CloseHandle(handle);
+    return true;
+  }
+  return false;
 }
 
 /////////////////////////////////////////////////
@@ -203,34 +207,44 @@ bool create_new_file_hardlink(const std::string &_hardlink,
   return ::CreateHardLinkA(_hardlink.c_str(), _target.c_str(), nullptr) == TRUE;
 }
 
-#endif
+#endif  // _WIN32
 
 #include <fstream> // NOLINT
+#include "ignition/common/Console.hh"
 #include "ignition/common/Filesystem.hh"
 
-namespace igncmn = ignition::common;
 using namespace ignition;
-using namespace igncmn;
+using namespace common;
+
+/// \brief Test Filesystem
+class FilesystemTest : public ::testing::Test
+{
+  // Documentation inherited
+  protected: void SetUp() override
+  {
+    Console::SetVerbosity(4);
+  }
+};
 
 /////////////////////////////////////////////////
 /// \brief Test file operations
-TEST(Util_TEST, fileOps)
+TEST_F(FilesystemTest, fileOps)
 {
-  EXPECT_FALSE(common::cwd().empty());
-  EXPECT_TRUE(common::exists(common::cwd()));
-  EXPECT_TRUE(common::isDirectory(common::cwd()));
-  EXPECT_FALSE(common::isFile(common::cwd()));
-
-  EXPECT_TRUE(common::isFile(__FILE__));
-  EXPECT_FALSE(common::isDirectory(__FILE__));
+  EXPECT_FALSE(cwd().empty());
+  EXPECT_TRUE(exists(cwd()));
+  EXPECT_TRUE(common::isDirectory(cwd()));
+  EXPECT_FALSE(isFile(cwd()));
 
   std::ofstream tmpOut("test.tmp");
   tmpOut << "Output" << std::endl;
   tmpOut.close();
 
-  EXPECT_TRUE(common::copyFile("test.tmp", "test2.tmp"));
-  EXPECT_TRUE(common::exists("test.tmp"));
-  EXPECT_TRUE(common::exists("test2.tmp"));
+  EXPECT_TRUE(isFile("test.tmp"));
+  EXPECT_FALSE(common::isDirectory("test.tmp"));
+
+  EXPECT_TRUE(copyFile("test.tmp", "test2.tmp"));
+  EXPECT_TRUE(exists("test.tmp"));
+  EXPECT_TRUE(exists("test2.tmp"));
 
   std::ifstream testIn("test.tmp");
   std::string testInContent((std::istreambuf_iterator<char>(testIn)),
@@ -246,10 +260,10 @@ TEST(Util_TEST, fileOps)
 
   EXPECT_EQ(testInContent, test2InContent);
 
-  EXPECT_TRUE(common::moveFile("test2.tmp", "test3.tmp"));
+  EXPECT_TRUE(moveFile("test2.tmp", "test3.tmp"));
 
-  EXPECT_FALSE(common::exists("test2.tmp"));
-  EXPECT_TRUE(common::exists("test3.tmp"));
+  EXPECT_FALSE(exists("test2.tmp"));
+  EXPECT_TRUE(exists("test3.tmp"));
 
   std::ifstream test3In("test3.tmp");
   std::string test3InContent((std::istreambuf_iterator<char>(test3In)),
@@ -261,43 +275,47 @@ TEST(Util_TEST, fileOps)
 
   EXPECT_EQ(testInContent, test3InContent);
 
-  EXPECT_FALSE(common::copyFile("test3.tmp", "test3.tmp"));
-  EXPECT_FALSE(common::copyFile("test3.tmp", "./test3.tmp"));
+  EXPECT_FALSE(copyFile("test3.tmp", "test3.tmp"));
+  EXPECT_FALSE(copyFile("test3.tmp", "./test3.tmp"));
 
   std::remove("test.tmp");
 
   // This file shouldn't exist, but we'll try to remove just in case the
   // test failed.
-  EXPECT_FALSE(common::removeFile("test2.tmp"));
+  EXPECT_FALSE(removeFile("test2.tmp"));
 
-  EXPECT_TRUE(common::removeFile("test3.tmp"));
+  EXPECT_TRUE(removeFile("test3.tmp"));
 
-  EXPECT_FALSE(common::exists("test3.tmp"));
+  EXPECT_FALSE(exists("test3.tmp"));
 }
 
 /////////////////////////////////////////////////
 /// \brief Test file operations
-TEST(Util_TEST, moreFileOps)
+TEST_F(FilesystemTest, moreFileOps)
 {
-  EXPECT_FALSE(common::copyFile("__wrong__.tmp", "test2.tmp"));
-  EXPECT_TRUE(!common::exists("test2.tmp"));
-  EXPECT_FALSE(common::copyFile("test.tmp", "__wrong_dir__/__wrong__.tmp"));
-  EXPECT_TRUE(!common::exists("__wrong_dir__"));
+  EXPECT_FALSE(copyFile("__wrong__.tmp", "test2.tmp"));
+  EXPECT_TRUE(!exists("test2.tmp"));
+  EXPECT_FALSE(copyFile("test.tmp", "__wrong_dir__/__wrong__.tmp"));
+  EXPECT_TRUE(!exists("__wrong_dir__"));
 
-  EXPECT_FALSE(common::moveFile("__wrong__.tmp", "test3.tmp"));
-  EXPECT_TRUE(!common::exists("test3.tmp"));
-  EXPECT_FALSE(common::moveFile("test2.tmp", "__wrong_dir__/__wrong__.tmp"));
-  EXPECT_TRUE(!common::exists("__wrong_dir__"));
+  EXPECT_FALSE(moveFile("__wrong__.tmp", "test3.tmp"));
+  EXPECT_TRUE(!exists("test3.tmp"));
+  EXPECT_FALSE(moveFile("test2.tmp", "__wrong_dir__/__wrong__.tmp"));
+  EXPECT_TRUE(!exists("__wrong_dir__"));
 }
 
 /////////////////////////////////////////////////
-TEST(Filesystem, exists)
+TEST_F(FilesystemTest, exists)
 {
   std::string new_temp_dir;
   ASSERT_TRUE(create_and_switch_to_temp_dir(new_temp_dir));
   ASSERT_TRUE(create_new_empty_file("newfile"));
   ASSERT_TRUE(createDirectory("fstestexists"));
 
+  const auto &relativeSubdir = joinPaths("fstestexists", "newfile2");
+  const auto &absoluteSubdir = joinPaths(new_temp_dir, relativeSubdir);
+  ASSERT_TRUE(create_new_empty_file(relativeSubdir));
+
   EXPECT_TRUE(exists("fstestexists"));
   EXPECT_TRUE(isDirectory("fstestexists"));
 
@@ -306,14 +324,24 @@ TEST(Filesystem, exists)
 
   EXPECT_TRUE(exists("newfile"));
   EXPECT_FALSE(isDirectory("newfile"));
+
+  // file in relative subdirectory
+  EXPECT_TRUE(exists(relativeSubdir));
+  EXPECT_TRUE(isFile(relativeSubdir));
+  EXPECT_FALSE(isDirectory(relativeSubdir));
+
+  // file in absolute subdirectory
+  EXPECT_TRUE(exists(absoluteSubdir));
+  EXPECT_TRUE(isFile(absoluteSubdir));
+  EXPECT_FALSE(isDirectory(absoluteSubdir));
 }
 
 // The symlink tests require special permissions to work on Windows,
 // so they will be disabled by default. For more information, see:
-// https://bitbucket.org/ignitionrobotics/ign-common/issues/21
+// https://github.com/ignitionrobotics/ign-common/issues/21
 #ifdef BUILD_SYMLINK_TESTS
 /////////////////////////////////////////////////
-TEST(Filesystem, symlink_exists)
+TEST_F(FilesystemTest, symlink_exists)
 {
   // There are 5 cases we want to test for links (Unix doesn't allow hard links
   // to directories or to non-existent files):
@@ -353,17 +381,22 @@ TEST(Filesystem, symlink_exists)
 #endif
 
 /////////////////////////////////////////////////
-TEST(Filesystem, cwd)
+TEST_F(FilesystemTest, cwd)
 {
+  const auto origCwd = cwd();
+
   std::string new_temp_dir;
   ASSERT_TRUE(create_and_switch_to_temp_dir(new_temp_dir));
 
   std::string path = cwd();
   EXPECT_EQ(path, new_temp_dir);
+
+  EXPECT_TRUE(chdir(origCwd));
+  EXPECT_EQ(origCwd, cwd());
 }
 
 /////////////////////////////////////////////////
-TEST(Filesystem, append)
+TEST_F(FilesystemTest, append)
 {
   std::string path = "tmp";
   path = joinPaths(path, "hello");
@@ -375,10 +408,75 @@ TEST(Filesystem, append)
   EXPECT_EQ(path, separator("tmp") +
             separator("hello") +
             separator("there") + "again");
+
+  path = joinPaths("base", "/before", "after/");
+
+#ifndef _WIN32
+  EXPECT_EQ(path, "base/before/after/");
+#else
+  EXPECT_EQ(path, "base\\before\\after\\");
+#endif
+
+  path = joinPaths("base", "/before", "after///");
+#ifndef _WIN32
+  EXPECT_EQ(path, "base/before/after/");
+#else
+  EXPECT_EQ(path, "base\\before\\after\\");
+#endif
+
+  path = joinPaths("///base", "/before", "after");
+#ifndef _WIN32
+  EXPECT_EQ(path, "/base/before/after");
+#else
+  EXPECT_EQ(path, "\\base\\before\\after");
+#endif
+
+  path = joinPaths("/base", "/before", "after");
+#ifndef _WIN32
+  EXPECT_EQ(path, "/base/before/after");
+#else
+  EXPECT_EQ(path, "\\base\\before\\after");
+#endif
+
+  path = joinPaths("///base", "///before//", "/after///");
+#ifndef _WIN32
+  EXPECT_EQ(path, "/base/before/after/");
+#else
+  EXPECT_EQ(path, "\\base\\before\\after\\");
+#endif
+
+  path = joinPaths("base", "/before", "after");
+#ifndef _WIN32
+  EXPECT_EQ(path, "base/before/after");
+#else
+  EXPECT_EQ(path, "base\\before\\after");
+#endif
+
+  // Make sure that the slashes in the middle of string are not altered.
+  path = joinPaths("https://fuel.ignitionrobotics.org", "/models", "box");
+  EXPECT_EQ(path, "https://fuel.ignitionrobotics.org/models/box");
 }
 
 /////////////////////////////////////////////////
-TEST(Filesystem, cwd_error)
+TEST_F(FilesystemTest, parentPath)
+{
+  std::string child = joinPaths("path", "to", "a", "child");
+  std::string parent = parentPath(child);
+  // Expect no separator at the end
+  EXPECT_EQ('a', *parent.rbegin());
+  // Expect parent string to match beginning of child string
+  EXPECT_EQ(parent, child.substr(0, parent.size()));
+  // join "a" with "child" and expect to match end of child string
+  EXPECT_EQ(separator("a") + "child", child.substr(parent.size()-1));
+
+  std::string child_with_slash = joinPaths("path", "to", "a", "child");
+  child_with_slash = separator(child_with_slash);
+  std::string parent2 = parentPath(child_with_slash);
+  EXPECT_EQ(parent, parent2);
+}
+
+/////////////////////////////////////////////////
+TEST_F(FilesystemTest, cwd_error)
 {
   // This test intentionally creates a directory, switches to it, removes
   // the directory, and then tries to call cwd() on it to cause
@@ -395,42 +493,75 @@ TEST(Filesystem, cwd_error)
 }
 
 /////////////////////////////////////////////////
-TEST(Filesystem, basename)
+TEST_F(FilesystemTest, decomposition)
 {
   std::string absolute = joinPaths("", "home", "bob", "foo");
   EXPECT_EQ(basename(absolute), "foo");
+#ifndef _WIN32
+   EXPECT_EQ(parentPath(absolute), "/home/bob");
+#else
+  EXPECT_EQ(parentPath(absolute), "home\\bob");
+#endif
 
   std::string relative = joinPaths("baz", "foobar");
   EXPECT_EQ(basename(relative), "foobar");
+  EXPECT_EQ(parentPath(relative), "baz");
 
   std::string bname = "bzzz";
   EXPECT_EQ(basename(bname), "bzzz");
+  EXPECT_EQ(parentPath(bname), "bzzz");
 
   std::string nobase = joinPaths("baz", "");
   EXPECT_EQ(basename(nobase), "baz");
+#ifndef _WIN32
+  EXPECT_EQ(parentPath(nobase), "baz/");
+#else
+  EXPECT_EQ(parentPath(nobase), "baz");
+#endif
 
   std::string multiple_slash = separator("baz") + separator("") + separator("")
     + separator("");
   EXPECT_EQ(basename(multiple_slash), "baz");
+#ifndef _WIN32
+  EXPECT_EQ(parentPath(multiple_slash), "baz//");
+#else
+  EXPECT_EQ(parentPath(multiple_slash), "baz\\\\");
+#endif
 
   std::string multiple_slash_middle = separator("") + separator("home")
     + separator("") + separator("") + separator("bob") + separator("foo");
   EXPECT_EQ(basename(multiple_slash_middle), "foo");
+#ifndef _WIN32
+  EXPECT_EQ(parentPath(multiple_slash_middle), "/home///bob");
+#else
+  EXPECT_EQ(parentPath(multiple_slash_middle), "\\home\\\\\\bob");
+#endif
 
   std::string multiple_slash_start = separator("") + separator("")
     + separator("") + separator("home") + separator("bob") + separator("foo");
   EXPECT_EQ(basename(multiple_slash_start), "foo");
+#ifndef _WIN32
+  EXPECT_EQ(parentPath(multiple_slash_start), "///home/bob");
+#else
+  EXPECT_EQ(parentPath(multiple_slash_start), "\\\\\\home\\bob");
+#endif
 
   std::string slash_only = separator("") + separator("");
   EXPECT_EQ(basename(slash_only), separator(""));
+  EXPECT_EQ(parentPath(slash_only), "");
 
   std::string multiple_slash_only = separator("") + separator("")
     + separator("") + separator("");
   EXPECT_EQ(basename(multiple_slash_only), separator(""));
+#ifndef _WIN32
+  EXPECT_EQ(parentPath(multiple_slash_only), "//");
+#else
+  EXPECT_EQ(parentPath(multiple_slash_only), "\\\\");
+#endif
 }
 
 /////////////////////////////////////////////////
-TEST(Filesystem, directory_iterator)
+TEST_F(FilesystemTest, directory_iterator)
 {
   std::string new_temp_dir;
   ASSERT_TRUE(create_and_switch_to_temp_dir(new_temp_dir));
@@ -452,14 +583,16 @@ TEST(Filesystem, directory_iterator)
     found_items.insert(basename(*dirIter));
   }
 
-  EXPECT_FALSE(found_items.find("newfile") == found_items.end());
-  EXPECT_FALSE(found_items.find("newdir") == found_items.end());
+  EXPECT_EQ(found_items.find("."), found_items.end());
+  EXPECT_EQ(found_items.find(".."), found_items.end());
+  EXPECT_NE(found_items.find("newfile"), found_items.end());
+  EXPECT_NE(found_items.find("newdir"), found_items.end());
 #ifdef BUILD_SYMLINK_TESTS
-  EXPECT_FALSE(found_items.find("symlink-file") == found_items.end());
-  EXPECT_FALSE(found_items.find("symlink-file-broken") == found_items.end());
-  EXPECT_FALSE(found_items.find("symlink-dir") == found_items.end());
-  EXPECT_FALSE(found_items.find("symlink-dir-broken") == found_items.end());
-  EXPECT_FALSE(found_items.find("hardlink-file") == found_items.end());
+  EXPECT_NE(found_items.find("symlink-file"), found_items.end());
+  EXPECT_NE(found_items.find("symlink-file-broken"), found_items.end());
+  EXPECT_NE(found_items.find("symlink-dir"), found_items.end());
+  EXPECT_NE(found_items.find("symlink-dir-broken"), found_items.end());
+  EXPECT_NE(found_items.find("hardlink-file"), found_items.end());
 #endif
 
   found_items.clear();
@@ -469,6 +602,142 @@ TEST(Filesystem, directory_iterator)
   }
 
   EXPECT_EQ(found_items.size(), 0UL);
+
+  found_items.clear();
+  for (DirIter dirIter("newDir"); dirIter != endIter; ++dirIter)
+  {
+    found_items.insert(basename(*dirIter));
+  }
+
+  EXPECT_EQ(found_items.size(), 0UL);
+}
+
+/////////////////////////////////////////////////
+TEST_F(FilesystemTest, createDirectories)
+{
+  std::string new_temp_dir;
+  ASSERT_TRUE(create_and_switch_to_temp_dir(new_temp_dir));
+
+  // test creating directories using relative path
+  std::string rel_dir = separator("rel_dir") + "subdir";
+  ASSERT_TRUE(createDirectories(rel_dir));
+  EXPECT_TRUE(exists(rel_dir));
+  EXPECT_TRUE(isDirectory(rel_dir));
+
+  // test creating directories using abs path
+  std::string path = cwd();
+  EXPECT_EQ(path, new_temp_dir);
+
+  std::string abs_dir = separator(cwd()) + separator("abs_dir") + "subdir";
+  ASSERT_TRUE(createDirectories(abs_dir));
+  EXPECT_TRUE(exists(abs_dir));
+  EXPECT_TRUE(isDirectory(abs_dir));
+}
+
+/////////////////////////////////////////////////
+TEST_F(FilesystemTest, copyDirectories)
+{
+  const auto origCwd = cwd();
+  std::string newTempDir;
+  ASSERT_TRUE(create_and_switch_to_temp_dir(newTempDir));
+
+  // Create an empty directory
+  std::string dirToBeCopied = "dir_to_be_copied";
+  EXPECT_FALSE(exists(dirToBeCopied));
+  EXPECT_TRUE(createDirectories(dirToBeCopied));
+  EXPECT_TRUE(exists(dirToBeCopied));
+  EXPECT_TRUE(isDirectory(dirToBeCopied));
+
+  // Copy to a directory
+  std::string dirCopied = "dir_copied";
+  EXPECT_FALSE(exists(dirCopied));
+
+  EXPECT_TRUE(copyDirectory(dirToBeCopied, dirCopied)) << dirToBeCopied <<
+      " -> " << dirCopied;
+  EXPECT_TRUE(exists(dirCopied));
+  EXPECT_TRUE(isDirectory(dirCopied));
+
+  // Copy to an existing directory - should overwrite
+  EXPECT_TRUE(copyDirectory(dirToBeCopied, dirCopied));
+  EXPECT_TRUE(exists(dirCopied));
+  EXPECT_TRUE(isDirectory(dirCopied));
+
+  // Copy to a directory with non-existent intermediate paths
+  std::string intermDirCopy = joinPaths("dir_interm", "dir_copied");
+  EXPECT_FALSE(exists(intermDirCopy));
+
+  EXPECT_TRUE(copyDirectory(dirToBeCopied, intermDirCopy));
+  EXPECT_TRUE(exists(intermDirCopy));
+  EXPECT_TRUE(isDirectory(intermDirCopy));
+
+  // Copy recursively a directory with a directory and a file in it
+  std::string subDir = joinPaths(dirToBeCopied, "sub_dir");
+  EXPECT_FALSE(exists(subDir));
+  EXPECT_TRUE(createDirectories(subDir));
+  EXPECT_TRUE(exists(subDir));
+
+  std::string subFile = joinPaths(subDir, "sub_file");
+  EXPECT_FALSE(exists(subFile));
+  EXPECT_TRUE(create_new_empty_file(subFile));
+  EXPECT_TRUE(exists(subFile));
+
+  std::string recDirCopied = "rec_dir_copied";
+  EXPECT_TRUE(copyDirectory(dirToBeCopied, recDirCopied));
+  EXPECT_TRUE(exists(recDirCopied));
+  EXPECT_TRUE(isDirectory(recDirCopied));
+
+  // Non-existent source directory
+  EXPECT_FALSE(copyDirectory("fake_dir", dirCopied));
+
+  // Cleanup
+  chdir(origCwd);
+  EXPECT_TRUE(removeAll(newTempDir));
+}
+
+/////////////////////////////////////////////////
+TEST_F(FilesystemTest, uniquePaths)
+{
+  std::string newTempDir;
+  ASSERT_TRUE(create_and_switch_to_temp_dir(newTempDir));
+
+  // Directory
+  std::string dir = "uniqueDirectory";
+  EXPECT_FALSE(exists(dir));
+  std::string dirRt = uniqueDirectoryPath(dir);
+  EXPECT_EQ(dir, dirRt);
+
+  ASSERT_TRUE(createDirectory(dir));
+  EXPECT_TRUE(exists(dir));
+  std::string dirExistingRt = uniqueDirectoryPath(dir);
+  EXPECT_EQ(dirExistingRt, dir + "(1)");
+
+  ASSERT_TRUE(createDirectory(dirExistingRt));
+  EXPECT_TRUE(exists(dirExistingRt));
+  std::string dirExistingRt2 = uniqueDirectoryPath(dir);
+  EXPECT_EQ(dirExistingRt2, dir + "(2)");
+
+  // File
+  std::string newFile = joinPaths(dir, "uniqueFile");
+  std::string newFileWithExt = newFile + ".txt";
+  EXPECT_FALSE(exists(newFileWithExt));
+  std::string fileRt = uniqueFilePath(newFile, "txt");
+  EXPECT_EQ(newFileWithExt, fileRt);
+
+  ASSERT_TRUE(create_new_empty_file(newFileWithExt));
+  EXPECT_TRUE(exists(newFileWithExt));
+  std::string fileExistingRt = uniqueFilePath(newFile, "txt");
+  EXPECT_EQ(fileExistingRt, newFile + "(1).txt");
+
+  ASSERT_TRUE(create_new_empty_file(fileExistingRt));
+  EXPECT_TRUE(exists(fileExistingRt));
+  std::string fileExistingRt2 = uniqueFilePath(newFile, "txt");
+  EXPECT_EQ(fileExistingRt2, newFile + "(2).txt");
+
+  // Cleanup
+  EXPECT_TRUE(removeFile(newFileWithExt)) << newFileWithExt;
+  EXPECT_TRUE(removeFile(fileExistingRt)) << fileExistingRt;
+  EXPECT_TRUE(removeDirectory(dir)) << dir;
+  EXPECT_TRUE(removeDirectory(dirExistingRt)) << dirExistingRt;
 }
 
 /////////////////////////////////////////////////
