@@ -59,10 +59,25 @@
 using namespace ignition;
 using namespace common;
 
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 Time Time::wallTime;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 
 struct timespec Time::clockResolution;
-const Time Time::Zero = common::Time(0, 0);
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+const Time Time::Zero = Time(0, 0);
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
+
 const int32_t Time::nsInSec = 1000000000L;
 const int32_t Time::nsInMs = 1000000;
 
@@ -71,6 +86,20 @@ const int32_t Time::nsInMs = 1000000;
 Time::Time()
   : sec(0), nsec(0)
 {
+#ifdef __MACH__
+  clockResolution.tv_sec = 1 / sysconf(_SC_CLK_TCK);
+#elif defined(_WIN32)
+  LARGE_INTEGER freq;
+  QueryPerformanceFrequency(&freq);
+  double period = 1.0/freq.QuadPart;
+  clockResolution.tv_sec = static_cast<int64_t>(floor(period));
+  clockResolution.tv_nsec =
+    static_cast<int64_t>((period - floor(period)) * nsInSec);
+#else
+  // get clock resolution, skip sleep if resolution is larger then
+  // requested sleep time
+  clock_getres(CLOCK_MONOTONIC, &clockResolution);
+#endif
 }
 
 /////////////////////////////////////////////////
@@ -85,13 +114,19 @@ Time::Time(const Time &_time)
 : sec(_time.sec), nsec(_time.nsec)
 {
 }
-
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 /////////////////////////////////////////////////
 Time::Time(int32_t _sec, int32_t _nsec)
 : sec(_sec), nsec(_nsec)
 {
   this->Correct();
 }
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 
 /////////////////////////////////////////////////
 Time::Time(double _time)
@@ -146,9 +181,16 @@ float Time::Float() const
 }
 
 /////////////////////////////////////////////////
-Time Time::Sleep(const common::Time &_time)
+Time Time::Sleep(const Time &_time)
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   Time result;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 
   if (_time >= clockResolution)
   {
@@ -160,7 +202,14 @@ Time Time::Sleep(const common::Time &_time)
     if (interval.tv_sec < 0)
     {
       ignerr << "Cannot sleep for negative time[" << _time << "]\n";
-      return result;
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+        return result;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
     }
 
     // This assert conforms to the manpage for nanosleep
@@ -168,7 +217,14 @@ Time Time::Sleep(const common::Time &_time)
     {
       ignerr << "Nanoseconds of [" << interval.tv_nsec
             << "] must be in the range0 to 999999999.\n";
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
       return result;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
     }
 
 #ifdef _WIN32
@@ -183,20 +239,41 @@ Time Time::Sleep(const common::Time &_time)
     if (timer == NULL)
     {
       ignerr << "Unable to create waitable timer. Sleep will be incorrect.\n";
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
       return result;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
     }
 
     if (!SetWaitableTimer (timer, &sleepTime, 0, NULL, NULL, 0))
     {
       ignerr << "Unable to use waitable timer. Sleep will be incorrect.\n";
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
       return result;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
     }
 
     if (WaitForSingleObject (timer, INFINITE) != WAIT_OBJECT_0)
     {
       ignerr <<
         "Unable to wait for a single object. Sleep will be incorrect.\n";
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
       return result;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
     }
 
     result.sec = 0;
@@ -204,18 +281,22 @@ Time Time::Sleep(const common::Time &_time)
 #else
     struct timespec remainder;
 # ifdef __MACH__
-    if (nanosleep(&interval, &remainder) == -1)
+    if (nanosleep(&interval, &remainder) == EINTR)
     {
+      // Get the time remaining (time unslept)
       result.sec = remainder.tv_sec;
       result.nsec = remainder.tv_nsec;
     }
 # else
-    if (clock_nanosleep(CLOCK_REALTIME, 0, &interval, &remainder) == -1)
+    if (clock_nanosleep(CLOCK_MONOTONIC, 0, &interval, &remainder) == EINTR)
     {
+      // Get the time remaining (time unslept)
       result.sec = remainder.tv_sec;
       result.nsec = remainder.tv_nsec;
     }
 # endif
+    // Compute the time actually slept.
+    result = _time - result;
 #endif
   }
   else
@@ -223,7 +304,14 @@ Time Time::Sleep(const common::Time &_time)
     ignlog << "Sleep time is larger than clock resolution, skipping sleep\n";
   }
 
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   return result;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 }
 
 /////////////////////////////////////////////////
@@ -238,10 +326,24 @@ Time &Time::operator=(const Time &_time)
 /////////////////////////////////////////////////
 Time Time::operator+(const Time &_time) const
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   Time t(this->sec + _time.sec, this->nsec + _time.nsec);
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
   t.Correct();
 
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   return t;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 }
 
 /////////////////////////////////////////////////
@@ -256,9 +358,16 @@ const Time &Time::operator+=(const Time &_time)
 /////////////////////////////////////////////////
 Time Time::operator-(const Time &_time) const
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   Time t(this->sec - _time.sec, this->nsec - _time.nsec);
   t.Correct();
   return t;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 }
 
 /////////////////////////////////////////////////
@@ -273,9 +382,16 @@ const Time &Time::operator-=(const Time &_time)
 /////////////////////////////////////////////////
 Time Time::operator*(const Time &_time) const
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   Time t(this->sec * _time.sec, this->nsec * _time.nsec);
   t.Correct();
   return t;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 }
 
 /////////////////////////////////////////////////
@@ -289,14 +405,26 @@ const Time &Time::operator*=(const Time &_time)
 /////////////////////////////////////////////////
 Time Time::operator/(const Time &_time) const
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   Time result(*this);
-
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
   if (_time.sec == 0 && _time.nsec == 0)
     ignerr << "Time divide by zero\n";
   else
     result.Set(this->Double() / _time.Double());
-
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   return result;
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 }
 
 /////////////////////////////////////////////////
@@ -315,7 +443,14 @@ bool Time::operator==(const Time &_time) const
 /////////////////////////////////////////////////
 bool Time::operator==(double _time) const
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   return *this == Time(_time);
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 }
 
 /////////////////////////////////////////////////
@@ -340,7 +475,14 @@ bool Time::operator<(const Time &_time) const
 /////////////////////////////////////////////////
 bool Time::operator<(double _time) const
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   return *this < Time(_time);
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 }
 
 /////////////////////////////////////////////////
@@ -352,7 +494,14 @@ bool Time::operator<=(const Time &_time) const
 /////////////////////////////////////////////////
 bool Time::operator<=(double _time) const
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   return *this <= Time(_time);
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 }
 
 /////////////////////////////////////////////////
@@ -364,7 +513,14 @@ bool Time::operator>(const Time &_time) const
 /////////////////////////////////////////////////
 bool Time::operator>(double _time) const
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   return *this > Time(_time);
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 }
 
 /////////////////////////////////////////////////
@@ -376,13 +532,27 @@ bool Time::operator>=(const Time &_time) const
 /////////////////////////////////////////////////
 bool Time::operator>=(double _time) const
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   return *this >= Time(_time);
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 }
 
 /////////////////////////////////////////////////
 bool Time::operator>=(const struct timespec &_tv) const
 {
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
   return *this >= Time(_tv);
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#endif
 }
 
 /////////////////////////////////////////////////
