@@ -471,7 +471,7 @@ void URIPath::PushFront(const std::string &_part)
     // all invalid characters should be
     // encoded, not just slashes.
     ignwarn << "Unencoded slashes in URI part, encoding them." << std::endl;
-    part = common::replaceAll(part, "/", "%2F");
+    part = replaceAll(part, "/", "%2F");
   }
 
   if (!part.empty())
@@ -508,7 +508,7 @@ void URIPath::PushBack(const std::string &_part)
     // all invalid characters should be
     // encoded, not just slashes.
     ignwarn << "Unencoded slashes in URI part, encoding them." << std::endl;
-    part = common::replaceAll(part, "/", "%2F");
+    part = replaceAll(part, "/", "%2F");
   }
 
   this->dataPtr->path.push_back(part);
@@ -587,6 +587,7 @@ URIPath &URIPath::operator=(const URIPath &_path)
 {
   this->dataPtr->path = _path.dataPtr->path;
   this->dataPtr->isAbsolute = _path.dataPtr->isAbsolute;
+  this->dataPtr->trailingSlash = _path.dataPtr->trailingSlash;
   return *this;
 }
 
@@ -659,14 +660,14 @@ bool URIPath::Parse(const std::string &_str)
 
   this->Clear();
 
-  for (auto part : common::split(_str, "/"))
+  for (auto part : split(_str, "/"))
     this->PushBack(part);
 
   // the initial / is removed from _str when splitting, so we need to
   // explicitly check for it
   this->dataPtr->isAbsolute = this->dataPtr->IsStringAbsolute(_str);
   this->dataPtr->trailingSlash =
-    _str.back() == '/' && _str.size() != 1;
+    _str.size() > 1 &&_str.back() == '/';
 
   return true;
 }
@@ -793,9 +794,9 @@ bool URIQuery::Parse(const std::string &_str)
 
   if (!_str.empty())
   {
-    for (const std::string &query : common::split(_str.substr(1), "&"))
+    for (const std::string &query : split(_str.substr(1), "&"))
     {
-      std::vector<std::string> values = common::split(query, "=");
+      std::vector<std::string> values = split(query, "=");
       if (values.size() == 2)
         this->Insert(values.at(0), values.at(1));
       else
@@ -1121,7 +1122,7 @@ bool URI::Valid(const std::string &_str)
 
     if (localScheme != "file" && authEndPos == authDelimPos+2)
     {
-      ignerr << "A host is manadatory when using a scheme other than file\n";
+      ignerr << "A host is mandatory when using a scheme other than file\n";
       return false;
     }
     else
@@ -1198,14 +1199,16 @@ bool URI::Parse(const std::string &_str)
       size_t authEndPos = str.find_first_of("/?#",
           authDelimPos + std::strlen(kAuthDelim));
 
+      if (authEndPos != std::string::npos &&
+          (str[authEndPos-1] == ':' && str[authEndPos] == '/'))
+      {
       // This could be a windows file path of the form file://D:/path/to/file
       // In this case, the authority is not present.
-      if (str[authEndPos-1] == ':' && str[authEndPos] == '/')
-      {
         str.erase(0, std::strlen(kAuthDelim));
         authorityPresent = false;
       }
-      else if (localScheme != "file" && authEndPos == authDelimPos+2)
+      else if (localScheme != "file" &&
+               authEndPos == authDelimPos + std::strlen(kAuthDelim))
       {
         ignerr << "A host is manadatory when using a scheme other than file\n";
         return false;
